from . import app_settings as defaults
from django.conf import settings


default_app_config = 'django_keycloak.apps.KeycloakAppConfig'

# Set some app default settings
for name in dir(defaults):
    if name.isupper() and not hasattr(settings, name):
        setattr(settings, name, getattr(defaults, name))



import mock


class MockTestCaseMixin(object):

    def __init__(self, *args, **kwargs):
        self._mocks = {}

        super(MockTestCaseMixin, self).__init__(*args, **kwargs)

    def setup_mock(self, target, autospec=True, **kwargs):
        if target in self._mocks:
            raise Exception('Target %s already patched', target)

        self._mocks[target] = mock.patch(target, autospec=autospec, **kwargs)

        return self._mocks[target].start()

    def tearDown(self):
        for mock_ in self._mocks.values():
            mock_.stop()

        return super(MockTestCaseMixin, self).tearDown()



import logging

from functools import partial

from django.utils import timezone

from django_keycloak.services.exceptions import TokensExpired

import django_keycloak.services.oidc_profile


logger = logging.getLogger(__name__)


def get_keycloak_id(client):
    """
    Get internal Keycloak id for client configured in Realm
    :param django_keycloak.models.Realm realm:
    :return:
    """
    keycloak_clients = client.admin_api_client.realms.by_name(
        name=client.realm.name).clients.all()
    for keycloak_client in keycloak_clients:
        if keycloak_client['clientId'] == client.client_id:
            return keycloak_client['id']

    return None


def get_authz_api_client(client):
    """
    :param django_keycloak.models.Client client:
    :rtype: keycloak.authz.KeycloakAuthz
    """
    return client.realm.realm_api_client.authz(client_id=client.client_id)


def get_openid_client(client):
    """
    :param django_keycloak.models.Client client:
    :rtype: keycloak.openid_connect.KeycloakOpenidConnect
    """
    openid = client.realm.realm_api_client.open_id_connect(
        client_id=client.client_id,
        client_secret=client.secret
    )

    if client.realm._well_known_oidc:
        openid.well_known.contents = client.realm.well_known_oidc

    return openid


def get_uma1_client(client):
    """
    :type client: django_keycloak.models.Client
    :rtype: keycloak.uma1.KeycloakUMA1
    """
    return client.realm.realm_api_client.uma1


def get_admin_client(client):
    """
    Get the Keycloak admin client configured for given realm.

    :param django_keycloak.models.Client client:
    :rtype: keycloak.admin.KeycloakAdmin
    """
    token = partial(get_access_token, client)
    return client.realm.realm_api_client.admin.set_token(token=token)


def get_service_account_profile(client):
    """
    Get service account for given client.

    :param django_keycloak.models.Client client:
    :rtype: django_keycloak.models.OpenIdConnectProfile
    """

    if client.service_account_profile:
        return client.service_account_profile

    token_response, initiate_time = get_new_access_token(client=client)

    oidc_profile = django_keycloak.services.oidc_profile._update_or_create(
        client=client,
        token_response=token_response,
        initiate_time=initiate_time)

    client.service_account_profile = oidc_profile
    client.save(update_fields=['service_account_profile'])

    return oidc_profile


def get_new_access_token(client):
    """
    Get client access_token

    :param django_keycloak.models.Client client:
    :rtype: str
    """
    scope = 'realm-management openid'

    initiate_time = timezone.now()
    token_response = client.openid_api_client.client_credentials(scope=scope)

    return token_response, initiate_time


def get_access_token(client):
    """
    Get access token from client's service account.
    :param django_keycloak.models.Client client:
    :rtype: str
    """

    oidc_profile = get_service_account_profile(client=client)

    try:
        return django_keycloak.services.oidc_profile.get_active_access_token(
            oidc_profile=oidc_profile)
    except TokensExpired:
        token_reponse, initiate_time = get_new_access_token(client=client)
        oidc_profile = django_keycloak.services.oidc_profile.update_tokens(
            token_model=oidc_profile,
            token_response=token_reponse,
            initiate_time=initiate_time
        )
        return oidc_profile.access_token



import logging

from django.utils import timezone

from django_keycloak.models import ExchangedToken

import django_keycloak.services.oidc_profile


logger = logging.getLogger(__name__)


def exchange_token(oidc_profile, remote_client):
    """
    Exchange access token from OpenID Connect profile for a token of given
    remote client.

    :param django_keycloak.models.OpenIdConnectProfile oidc_profile:
    :param django_keycloak.models.RemoteClient remote_client:
    :rtype: dict
    """
    active_access_token = django_keycloak.services.oidc_profile\
        .get_active_access_token(oidc_profile=oidc_profile)

    # http://www.keycloak.org/docs/latest/securing_apps/index.html#_token-exchange
    return oidc_profile.realm.client.openid_api_client.token_exchange(
        audience=remote_client.name,
        subject_token=active_access_token,
        requested_token_type='urn:ietf:params:oauth:token-type:refresh_token'
    )


def get_active_remote_client_token(oidc_profile, remote_client):
    """
    Get an active remote client token. Exchange when not available or expired.

    :param django_keycloak.models.OpenIdConnectProfile oidc_profile:
    :param django_keycloak.models.RemoteClient remote_client:
    :rtype: str
    """
    exchanged_token, _ = ExchangedToken.objects.get_or_create(
        oidc_profile=oidc_profile,
        remote_client=remote_client
    )

    initiate_time = timezone.now()

    if exchanged_token.refresh_expires_before is None \
            or initiate_time > exchanged_token.refresh_expires_before \
            or initiate_time > exchanged_token.expires_before:
        token_response = exchange_token(oidc_profile, remote_client)

        exchanged_token = django_keycloak.services.oidc_profile.update_tokens(
            token_model=exchanged_token,
            token_response=token_response,
            initiate_time=initiate_time
        )

    return exchanged_token.access_token



import base64


def credential_representation_from_hash(hash_, temporary=False):
    algorithm, hashIterations, salt, hashedSaltedValue = hash_.split('$')

    return {
        'type': 'password',
        'hashedSaltedValue': hashedSaltedValue,
        'algorithm': algorithm.replace('_', '-'),
        'hashIterations': int(hashIterations),
        'salt': base64.b64encode(salt.encode()).decode('ascii').strip(),
        'temporary': temporary
    }


def add_user(client, user):
    """
    Create user in Keycloak based on a local user including password.

    :param django_keycloak.models.Client client:
    :param django.contrib.auth.models.User user:
    """
    credentials = credential_representation_from_hash(hash_=user.password)

    client.admin_api_client.realms.by_name(client.realm.name).users.create(
        username=user.username,
        credentials=credentials,
        first_name=user.first_name,
        last_name=user.last_name,
        email=user.email,
        enabled=user.is_active
    )



from __future__ import unicode_literals

import logging

from django.core.management.base import BaseCommand
from django.contrib.auth import get_user_model

from django_keycloak.models import Realm

import django_keycloak.services.users

logger = logging.getLogger(__name__)


def realm(name):
    try:
        return Realm.objects.get(name=name)
    except Realm.DoesNotExist:
        raise TypeError('Realm does not exist')


def user(username):
    UserModel = get_user_model()
    try:
        return UserModel.objects.get(username=username)
    except UserModel.DoesNotExist:
        raise TypeError('User does not exist')


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('--realm', type=realm, required=True)
        parser.add_argument('--user', type=user, required=True)

    def handle(self, *args, **options):
        user = options['user']
        realm = options['realm']

        django_keycloak.services.users.add_user(client=realm.client, user=user)
